//@author: vux
//@help: template for simple compute shader (buffer write)
//@tags: template
//@credits: 


struct MeshPoint
{
	float3 pos; 
	float3 normal;
	float4 tangent;
	float2 uv;
	
	int3 tanInt1;
	int3 tanInt2;
	int3 normalInt;
};

static float PI = 3.14159265359;

//Buffers
StructuredBuffer <MeshPoint> meshVBuffIn;
RWStructuredBuffer <MeshPoint> meshVBuff;
RWStructuredBuffer <uint> trianglesBuff;
RWStructuredBuffer <float3> meshNormalsBuff;
RWStructuredBuffer <float3> idBuff;

[numthreads(32,1,1)]
void ComputeFaceNormals (uint3 id : SV_DispatchThreadID)
{
	uint cID = id.x * 3;
	
	meshVBuff[cID] = meshVBuffIn[cID]; 
	//Smooth surface normal calculation. 
	//Accumulate Normals and normalize in vertex shader.
	//https://www.iquilezles.org/www/articles/normals/normals.htm
	//https://www.bytehazard.com/articles/vertnorm.html - utile
	uint ia = trianglesBuff[cID];
	uint ib = trianglesBuff[cID+1];
	uint ic = trianglesBuff[cID+2];
	float3 v1 = meshVBuff[ia].pos;
	float3 v2 = meshVBuff[ib].pos;
	float3 v3 = meshVBuff[ic].pos;
	float3 e1 = v2 - v1;
	float3 e2 = v3 - v1;
	float3 norm  = cross( e1, e2 );

	//norm = id.x == 10000 ? float3(0,1,0) : norm;
	
	//Float normals to int normals
	int3 normI = (int3)(1000000 * norm);
	
	//ok cos ci siamo: perdo un po' di precisione castando a int
	//ma non  un gran problema in questo caso
	InterlockedAdd(meshVBuff[ia].normalInt.x, normI.x);
	InterlockedAdd(meshVBuff[ia].normalInt.y, normI.y);
	InterlockedAdd(meshVBuff[ia].normalInt.z, normI.z);
	InterlockedAdd(meshVBuff[ib].normalInt.x, normI.x);
	InterlockedAdd(meshVBuff[ib].normalInt.y, normI.y);
	InterlockedAdd(meshVBuff[ib].normalInt.z, normI.z);
	InterlockedAdd(meshVBuff[ic].normalInt.x, normI.x);
	InterlockedAdd(meshVBuff[ic].normalInt.y, normI.y);
	InterlockedAdd(meshVBuff[ic].normalInt.z, normI.z);

	//TANGENTS
	float2 tc1 = meshVBuff[ia].uv;
    float2 tc2 = meshVBuff[ib].uv;
    float2 tc3 = meshVBuff[ic].uv;

	float s1 = tc2.x - tc1.x;
	float s2 = tc3.x - tc1.x;
	float t1 = tc2.y - tc1.y;
	float t2 = tc3.y - tc1.y;

	float r = 1.0 / (s1 * t2 - s2 * t1);
	
	int3 sdir = (int3) (1000000 *  ( r * ((t2 * e1) - (t1 * e2)) ) );//float3 sdir = r * ((t2 * e1) - (t1*e2));
	int3 tdir = (int3) (1000000 *  ( r * ((s1 * e2) - (s2 * e1)) ) );

	InterlockedAdd(meshVBuff[ia].tanInt1.x, sdir.x);
	InterlockedAdd(meshVBuff[ia].tanInt1.y, sdir.y);
	InterlockedAdd(meshVBuff[ia].tanInt1.z, sdir.z);
	InterlockedAdd(meshVBuff[ib].tanInt1.x, sdir.x);
	InterlockedAdd(meshVBuff[ib].tanInt1.y, sdir.y);
	InterlockedAdd(meshVBuff[ib].tanInt1.z, sdir.z);
	InterlockedAdd(meshVBuff[ic].tanInt1.x, sdir.x);
	InterlockedAdd(meshVBuff[ic].tanInt1.y, sdir.y);
	InterlockedAdd(meshVBuff[ic].tanInt1.z, sdir.z);

	InterlockedAdd(meshVBuff[ia].tanInt2.x, tdir.x);
	InterlockedAdd(meshVBuff[ia].tanInt2.y, tdir.y);
	InterlockedAdd(meshVBuff[ia].tanInt2.z, tdir.z);
	InterlockedAdd(meshVBuff[ib].tanInt2.x, tdir.x);
	InterlockedAdd(meshVBuff[ib].tanInt2.y, tdir.y);
	InterlockedAdd(meshVBuff[ib].tanInt2.z, tdir.z);
	InterlockedAdd(meshVBuff[ic].tanInt2.x, tdir.x);
	InterlockedAdd(meshVBuff[ic].tanInt2.y, tdir.y);
	InterlockedAdd(meshVBuff[ic].tanInt2.z, tdir.z);
}

technique11 Apply
{
	pass P0
	{
		SetComputeShader( CompileShader( cs_5_0, ComputeFaceNormals() ) );
	}
}




