//@author: ondeowl
//@help: template for simple compute shader (buffer write)
//@tags: template
//@credits: 

  
struct MeshPoint
{
	float3 pos; 
	float3 normal;
	float4 tangent;
	float2 uv;
	
	int3 tanInt1;
	int3 tanInt2;
	int3 normalInt;
};

static float PI = 3.14159265359;
//Buffers
StructuredBuffer <MeshPoint> meshVBuffIn;
RWStructuredBuffer <MeshPoint> meshVBuff;
RWStructuredBuffer <uint> trianglesBuff;
RWStructuredBuffer <float3> meshNormalsBuff;
RWStructuredBuffer <float3> idBuff;

uint myModuloOp(uint a, uint b)
{
	return a - (b * floor(a / b));
}

float3 safeNormalize(float3 v)
{
	if(length(v)>0)
	{
		return normalize(v);
	}
	return float3(0,1,0);
}


[numthreads(32,1,1)]
void SolveTangents (uint3 id : SV_DispatchThreadID)
{
	//https://forum.unity.com/threads/how-to-calculate-mesh-tangents.38984/
	uint i = id.x;
	meshVBuff[i] = meshVBuffIn[i];
	float3 n = (float3) (meshVBuff[i].normalInt) / 1000000.0; // INT BACK TO FLOAT
	float3 t = (float3) (meshVBuff[i].tanInt1) / 1000000.0;
	float3 t2 = (float3) (meshVBuff[i].tanInt2) / 1000000.0;

	// Gram-Schmidt orthogonalize
    float3 finalT = safeNormalize(t.xyz - n * dot(n, t.xyz));
	
	meshVBuff[i].normal = n;
	meshVBuff[i].tangent.xyz = finalT.xyz;
	
	// Calculate handedness
    meshVBuff[i].tangent.w = (dot(cross(n, t), t2) > 0) ? -1 : 1;// * 2 - 1; //? -1.0f : 1.0f;

}

technique11 Apply
{
	pass P0
	{
		SetComputeShader( CompileShader( cs_5_0, SolveTangents() ) );
	}
}




